@@ -1,4 +1,4 @@
-// Copyright (C) 2016  Stefan Vargyas
+// Copyright (C) 2016, 2017  Stefan Vargyas
 // 
 // This file is part of Json-Type.
 // 
@@ -138,10 +138,12 @@ enum json_type_ruler_error_type_t
     json_type_ruler_error_empty_obj_not_allowed,
     json_type_ruler_error_invalid_obj_first_key,
     json_type_ruler_error_invalid_type_obj_size_not_two,
+    json_type_ruler_error_invalid_type_obj_size_not_two_or_three,
     json_type_ruler_error_invalid_name_obj_size_not_two,
     json_type_ruler_error_invalid_plain_obj_size_not_one,
     json_type_ruler_error_invalid_type_obj_type_not_obj_arr_list_dict,
     json_type_ruler_error_invalid_type_obj_2nd_arg_not_args,
+    json_type_ruler_error_invalid_type_obj_3rd_arg_not_expr,
     json_type_ruler_error_invalid_name_obj_2nd_arg_not_type,
     json_type_ruler_error_invalid_plain_obj_type_not_null_bool_num_str,
     json_type_ruler_error_invalid_name_obj_name_not_str,
@@ -149,9 +151,14 @@ enum json_type_ruler_error_type_t
     json_type_ruler_error_invalid_object_obj_args_not_array_of_name_objs,
     json_type_ruler_error_invalid_list_obj_args_not_of_type_array_of_types,
     json_type_ruler_error_invalid_dict_obj_args_not_array_of_name_objs,
+    json_type_ruler_error_invalid_dict_obj_expr_not_string,
     json_type_ruler_error_invalid_array_obj_args_neither_type_nor_array_of_types,
     json_type_ruler_error_invalid_array_elem_is_neither_type_nor_name_obj,
-    json_type_ruler_error_invalid_array_elem_is_either_type_xor_name_obj
+    json_type_ruler_error_invalid_array_elem_is_either_type_xor_name_obj,
+    json_type_ruler_error_invalid_dict_expr_unexpected_char,
+    json_type_ruler_error_invalid_dict_expr_unexpected_token,
+    json_type_ruler_error_invalid_dict_expr_unexpected_end_of_expr,
+    json_type_ruler_error_invalid_dict_expr_empty_quoted_key,
 };
 
 struct json_type_lib_error_meta_t
@@ -179,6 +186,7 @@ enum json_type_lib_error_attr_type_t
     json_type_lib_error_attr_invalid_list_elem_is_list,
     json_type_lib_error_attr_invalid_list_elem_is_dict,
     json_type_lib_error_attr_invalid_dict_dup_key_name,
+    json_type_lib_error_attr_invalid_dict_expr_key_not_found,
     json_type_lib_error_attr_invalid_defs_dup_name,
 };
 
@@ -217,6 +225,7 @@ enum json_type_check_error_type_t
     json_type_check_error_too_few_args,
     json_type_check_error_invalid_arg_name,
     json_type_check_error_duplicate_arg_name,
+    json_type_check_error_dict_expr_falsified,
 };
 
 enum json_type_check_error_key_arg_type_t
@@ -244,6 +253,13 @@ struct json_type_check_error_key_arg_t
     };
 };
 
+struct json_type_check_error_bit_arg_t
+{
+    const struct json_type_dict_node_t* node;
+    const struct bit_set_t*             args;
+    const struct json_type_dict_attr_t* attr;
+};
+
 enum json_type_check_error_val_arg_type_t
 {
     json_type_check_error_val_arg_node_type,
@@ -275,7 +291,8 @@ enum json_type_check_error_arg_type_t
 {
     json_type_check_error_arg_none_type,
     json_type_check_error_arg_key_type,
-    json_type_check_error_arg_val_type
+    json_type_check_error_arg_bit_type,
+    json_type_check_error_arg_val_type,
 };
 
 struct json_type_check_error_arg_t
@@ -284,6 +301,7 @@ struct json_type_check_error_arg_t
     union {
         // ...                                 none;
         struct json_type_check_error_key_arg_t key;
+        struct json_type_check_error_bit_arg_t bit;
         struct json_type_check_error_val_arg_t val;
     };
 };
