@@ -69,10 +69,13 @@ struct json_esc_t* json_esc_create(
     // the STATIC assertion below; this property
     // is checked only once in this function
     STATIC(
-        json_validate_utf8_config + 1 ==
+        json_allow_surrogate_pairs_config + 1 <=
+        json_validate_utf8_config &&
+
+        json_validate_utf8_config + 1 <=
         json_trim_spaces_config &&
 
-        json_trim_spaces_config + 1 ==
+        json_trim_spaces_config + 1 <=
         json_quote_text_config);
 
     JSON_ESC_SIZES_VALIDATE(sizes);
@@ -148,11 +151,11 @@ static const uchar_t json_esc_escapes[ascii_size] = {
     )
 #define JSON_ESC_CURRENT_STATUS() (esc->status)
 
-#define JSON_ESC_CURRENT_CONF(p)               \
-    (                                          \
-        (esc->current & (SZ(1) <<              \
-            (json_ ## p ## _config -           \
-             json_validate_utf8_config))) != 0 \
+#define JSON_ESC_CURRENT_CONF(p)                       \
+    (                                                  \
+        (esc->current & (SZ(1) <<                      \
+            (json_ ## p ## _config -                   \
+             json_allow_surrogate_pairs_config))) != 0 \
     )
 
 #define JSON_ESC_ERROR_(t, p)                   \
@@ -401,15 +404,17 @@ enum json_parse_status_t json_esc_do_parse(
             esc->contn = 0;
         }
         else {
-            esc->point <<= SZ(6);
-            esc->point |= *buf & ~0xc0u;
-
             ASSERT(esc->n_utf8 < ARRAY_SIZE(esc->utf8));
             esc->utf8[esc->n_utf8 ++] = *buf;
 
             ASSERT_SIZE_DEC_NO_OVERFLOW(esc->contn);
             esc->contn --;
 
+            if (JSON_ESC_CURRENT_CONF(validate_utf8)) {
+                esc->point <<= SZ(6);
+                esc->point |= *buf & ~0xc0u;
+            }
+
             if (JSON_ESC_CURRENT_CONF(validate_utf8) &&
                 esc->contn == 0) {
                 size_t e = 0;
@@ -440,11 +445,16 @@ enum json_parse_status_t json_esc_do_parse(
                 // been rejected by json_validate_utf8 above
                 ASSERT(esc->point < 0xd800u || esc->point > 0xdfffu);
 
-                // stev: until 'json_obj' is rejecting surrogate
-                // code points, refrain to generate them here
-                if (esc->point >= 0x10000u)
-                    return JSON_ESC_ERROR_PT(non_bmp_utf8);
+                if (esc->point >= 0x10000u) {
+                    if (!JSON_ESC_CURRENT_CONF(allow_surrogate_pairs))
+                        return JSON_ESC_ERROR_PT(non_bmp_utf8);
 
+                    esc->point -= 0x10000u;
+                    printf("\\u%04zx", 0xd800u | (esc->point >> 10));
+
+                    esc->point &= 0x3ffu;
+                    esc->point |= 0xdc00u;
+                }
                 printf("\\u%04zx", esc->point);
                 esc->printed = true;
             }
@@ -490,10 +500,11 @@ bool json_esc_config_get_param(
     struct json_esc_t* esc, enum json_config_param_t param)
 {
     switch (param) {
+    case json_allow_surrogate_pairs_config:
     case json_validate_utf8_config:
     case json_trim_spaces_config:
     case json_quote_text_config:
-        param -= json_validate_utf8_config;
+        param -= json_allow_surrogate_pairs_config;
         return esc->config & (SZ(1) << param);
     default:
         INVALID_ARG("%d", param);
@@ -505,10 +516,11 @@ void json_esc_config_set_param(
     bool val)
 {
     switch (param) {
+    case json_allow_surrogate_pairs_config:
     case json_validate_utf8_config:
     case json_trim_spaces_config:
     case json_quote_text_config:
-        param -= json_validate_utf8_config;
+        param -= json_allow_surrogate_pairs_config;
         if (val)
             esc->config |= (SZ(1) << param);
         else
