@@ -44,14 +44,27 @@ const char* json_build_datetime(void)
     return __DATE__ " " __TIME__;
 }
 
-static bool json_encode_utf8(uchar_t* buf, size_t* len)
+enum json_code_point_type_t
+{
+    json_basic_plane_code_point,
+    json_high_surrogate_code_point,
+    json_low_surrogate_code_point
+};
+
+struct json_surrogate_pair_t
+{
+    uint16_t high;
+    uint16_t low;
+};
+
+static enum json_code_point_type_t json_encode_utf8(
+    struct json_surrogate_pair_t* surrogates,
+    uchar_t* buf, size_t* len)
 {
-    typedef unsigned int uint_t;
     uchar_t* p = buf;
-    uint_t c = 0;
+    uint16_t c = 0;
 
     STATIC(TYPE_WIDTH(uchar_t) >= 8);
-    STATIC(TYPE_WIDTH(uint_t) >= 16);
 
     for (; p < buf + 4; p ++) {
         ASSERT((*p & 0xf0u) == 0);
@@ -64,10 +77,14 @@ static bool json_encode_utf8(uchar_t* buf, size_t* len)
     // The Unicode Standard Version 8.0 - Core Specification, Chapter 3, p. 125
     // http://www.unicode.org/versions/Unicode8.0.0/ch03.pdf
 
-    // stev: TODO: decode surrogate code point pairs to non-BMP
-    // 4-byte UTF-8 (the test below rejects such code points)
-    if (c >= 0xd800u && c <= 0xdfffu)
-        return false;
+    if (c >= 0xd800u && c <= 0xdbffu) {
+        surrogates->high = c;
+        return json_high_surrogate_code_point;
+    }
+    if (c >= 0xdc00u && c <= 0xdfffu) {
+        surrogates->low = c;
+        return json_low_surrogate_code_point;
+    }
 
     p = buf;
     if (c < 0x80u)
@@ -77,21 +94,37 @@ static bool json_encode_utf8(uchar_t* buf, size_t* len)
         *p ++ = (uchar_t) ((c >> 6) | 0xc0u);
         *p ++ = (uchar_t) ((c & 0x3f) | 0x80u);
     }
-    else
-    /* if (c < 0x10000u) */ {
+    else {
         *p ++ = (uchar_t) ((c >> 12) | 0xe0u);
         *p ++ = (uchar_t) (((c >> 6) & 0x3fu) | 0x80u);
         *p ++ = (uchar_t) ((c & 0x3fu) | 0x80u);
     }
-    // else {
-    // 	*p ++ = (uchar_t) ((c >> 18) | 0xf0u);
-    // 	*p ++ = (uchar_t) (((c >> 12) & 0x3fu) | 0x80u);
-    // 	*p ++ = (uchar_t) (((c >> 6) & 0x3fu) | 0x80u);
-    // 	*p ++ = (uchar_t) ((c & 0x3fu) | 0x80u);
-    // }
     *len = PTR_DIFF(p, buf);
 
-    return true;
+    return json_basic_plane_code_point;
+}
+
+static void json_encode_utf8_surrogate_pair(
+    const struct json_surrogate_pair_t* surrogates, uchar_t* buf)
+{
+    uint32_t c;
+
+    ASSERT(
+        surrogates->high >= 0xd800u &&
+        surrogates->high <= 0xdbffu);
+    ASSERT(
+        surrogates->low  >= 0xdc00u &&
+        surrogates->low  <= 0xdfffu);
+
+    c = surrogates->high & 0x3ffu;
+    c <<= 10;
+    c += surrogates->low & 0x3ffu;
+    c += 0x10000u;
+
+    *buf ++ = (uchar_t) ((c >> 18) | 0xf0u);
+    *buf ++ = (uchar_t) (((c >> 12) & 0x3fu) | 0x80u);
+    *buf ++ = (uchar_t) (((c >> 6) & 0x3fu) | 0x80u);
+    *buf ++ = (uchar_t) ((c & 0x3fu) | 0x80u);
 }
 
 bool json_validate_utf8(const uchar_t* buf, size_t len, size_t* err)
@@ -358,10 +391,16 @@ enum json_token_string_state_t
     json_token_string_start_state,
     json_token_string_plain_char_state,
     json_token_string_esc_char_state,
+    json_token_string_surrogate_state,
+    json_token_string_esc_char2_state,
     json_token_string_hexa1_state,
     json_token_string_hexa2_state,
     json_token_string_hexa3_state,
     json_token_string_hexa4_state,
+    json_token_string_hexa5_state,
+    json_token_string_hexa6_state,
+    json_token_string_hexa7_state,
+    json_token_string_hexa8_state,
     json_token_string_error_state,
 };
 
@@ -377,10 +416,16 @@ static const char* json_token_string_state_get_name(
         CASE_STATE(start),
         CASE_STATE(plain_char),
         CASE_STATE(esc_char),
+        CASE_STATE(surrogate),
+        CASE_STATE(esc_char2),
         CASE_STATE(hexa1),
         CASE_STATE(hexa2),
         CASE_STATE(hexa3),
         CASE_STATE(hexa4),
+        CASE_STATE(hexa5),
+        CASE_STATE(hexa6),
+        CASE_STATE(hexa7),
+        CASE_STATE(hexa8),
         CASE_STATE(error)
     };
     return ARRAY_NON_NULL_ELEM(names, state);
@@ -391,6 +436,7 @@ static const char* json_token_string_state_get_name(
 struct json_token_string_part_t
 {
     enum json_token_string_state_t state;
+    struct json_surrogate_pair_t surrogates; 
     size_t pos;
 };
 
@@ -553,8 +599,11 @@ static void json_token_string_part_print_debug(
         PART_VALUE(string);
 
     print_debug_fmt(
-        "{.state=%s .pos=%zu}", json_token_string_state_get_name(
-            val->state), val->pos);
+        "{.state=%s .surrogates={.high=0x%zx .low=0x%zx} .pos=%zu}",
+            json_token_string_state_get_name(val->state),
+            UINT_TO_SIZE(val->surrogates.high),
+            UINT_TO_SIZE(val->surrogates.low),
+            val->pos);
 }
 
 #define CASE_PART2(n, f) [json_token_ ## n ## _part] = { .name = #n, .func = f }
@@ -675,27 +724,44 @@ static inline void json_mem_buf_append(struct json_mem_buf_t* buf,
 
 #define JSON_MEM_BUF_APPEND(buf, ch) MEM_BUF_APPEND(buf, ch)
 
-static bool json_mem_buf_encode_utf8(struct json_mem_buf_t* buf,
-    const uchar_t* ptr)
+static enum json_code_point_type_t json_mem_buf_encode_utf8(
+    struct json_mem_buf_t* buf,
+    struct json_surrogate_pair_t* surrogates,
+    size_t pos)
 {
+    enum json_code_point_type_t r;
     size_t w;
 
     // stev: main invariant of buf
     ASSERT(buf->size >= buf->len);
 
-    // stev: ptr is inside buf and
-    // at four bytes from the end
-    ASSERT(buf->len >= 4);
-    ASSERT(buf->ptr <= ptr);
-    ASSERT(buf->ptr + buf->size > ptr);
-    ASSERT(buf->ptr + buf->len == ptr + 4);
+    // stev: pos is inside buf, at 4 bytes from the end
+    ASSERT(buf->len == SIZE_ADD(pos, SZ(4)));
 
-    if (json_encode_utf8(CONST_CAST(ptr, uchar_t), &w)) {
-        ASSERT(w <= 4U && w > 0);
-        buf->len -= 4U - w;
-        return true;
+    STATIC(json_basic_plane_code_point == 0);
+
+    if (!(r = json_encode_utf8(
+                surrogates, buf->ptr + pos, &w))) {
+        ASSERT(w <= SZ(4) && w > 0);
+        buf->len -= SZ(4) - w;
     }
-    return false;
+
+    return r;
+}
+
+static void json_mem_buf_encode_utf8_surrogate_pair(
+    struct json_mem_buf_t* buf,
+    const struct json_surrogate_pair_t* surrogates,
+    size_t pos)
+{
+    // stev: main invariant of buf
+    ASSERT(buf->size >= buf->len);
+
+    // stev: pos is inside buf, at 8 bytes from the end
+    ASSERT(buf->len == SIZE_ADD(pos, SZ(8)));
+
+    json_encode_utf8_surrogate_pair(surrogates, buf->ptr + pos);
+    buf->len -= SZ(4);
 }
 
 static inline bool json_mem_buf_validate_utf8(struct json_mem_buf_t* buf,
@@ -818,7 +884,7 @@ static bool json_parse_number_token(
 
     ASSERT(muf->size >= muf->len);
 
-    while (buf->ptr < buf->end && !done) {
+    while (buf->ptr < buf->end) {
         uchar_t ch = *buf->ptr;
 
     next_state:
@@ -1032,6 +1098,7 @@ static bool json_parse_string_token(
         PRINT_PARSE_TOKEN_BUF(string, 10);
 
         switch (part->state) {
+
         case JSON_STR_STATE(start):
             ASSERT(muf->len == 0);
             ASSERT(ch == '"');
@@ -1074,15 +1141,29 @@ static bool json_parse_string_token(
             }
             break;
 
-        case JSON_STR_STATE(hexa1):
-        case JSON_STR_STATE(hexa2):
-        case JSON_STR_STATE(hexa3):
-        case JSON_STR_STATE(hexa4):
+        case JSON_STR_STATE(surrogate):
+            if (ch != '\\') {
+                part->state = JSON_STR_STATE(error);
+                goto loop_out;
+            }
+            part->state = JSON_STR_STATE(esc_char2);
+            break;
+
+        case JSON_STR_STATE(esc_char2):
+            if (ch != 'u') {
+                part->state = JSON_STR_STATE(error);
+                goto loop_out;
+            }
+            part->state = JSON_STR_STATE(hexa5);
+            break;
+
+        case JSON_STR_STATE(hexa1) ... JSON_STR_STATE(hexa8):
             if (!IS_ASCII(ch) || (ch = hexa[ch]) == 0) {
                 part->state = JSON_STR_STATE(error);
                 goto loop_out;
             }
-            if (part->state == JSON_STR_STATE(hexa1))
+            if (part->state == JSON_STR_STATE(hexa1) ||
+                part->state == JSON_STR_STATE(hexa5))
                 part->pos = muf->len;
 
             ch &= ~ hex_bit;
@@ -1091,17 +1172,48 @@ static bool json_parse_string_token(
             STATIC(
                 JSON_STR_STATE(hexa1) + 1 == JSON_STR_STATE(hexa2) &&
                 JSON_STR_STATE(hexa2) + 1 == JSON_STR_STATE(hexa3) &&
-                JSON_STR_STATE(hexa3) + 1 == JSON_STR_STATE(hexa4));
-            if (part->state == JSON_STR_STATE(hexa4)) {
-                if (!json_mem_buf_encode_utf8(
-                        muf, muf->ptr + part->pos)) {
-                    part->state = JSON_STR_STATE(error);
-                    goto loop_out;
-                }
-                part->state = JSON_STR_STATE(plain_char);
-            }
-            else
+                JSON_STR_STATE(hexa3) + 1 == JSON_STR_STATE(hexa4) &&
+                JSON_STR_STATE(hexa4) + 1 == JSON_STR_STATE(hexa5) &&
+                JSON_STR_STATE(hexa5) + 1 == JSON_STR_STATE(hexa6) &&
+                JSON_STR_STATE(hexa6) + 1 == JSON_STR_STATE(hexa7) &&
+                JSON_STR_STATE(hexa7) + 1 == JSON_STR_STATE(hexa8));
+
+            if (part->state != JSON_STR_STATE(hexa4) &&
+                part->state != JSON_STR_STATE(hexa8))
                 part->state ++;
+            else {
+                enum json_code_point_type_t t;
+
+                t = json_mem_buf_encode_utf8(
+                        muf, &part->surrogates, part->pos);
+
+                if (part->state == JSON_STR_STATE(hexa4)) {
+                    if (t == json_basic_plane_code_point)
+                        part->state = JSON_STR_STATE(plain_char);
+                    else
+                    if (t == json_high_surrogate_code_point &&
+                        JSON_OBJECT_CONF(allow_surrogate_pairs))
+                        part->state = JSON_STR_STATE(surrogate);
+                    else {
+                        part->state = JSON_STR_STATE(error);
+                        goto loop_out;
+                    }
+                }
+                else { // part->state == JSON_STR_STATE(hexa8)
+                    if (t == json_low_surrogate_code_point) {
+                        json_mem_buf_encode_utf8_surrogate_pair(
+                            muf,
+                            &part->surrogates,
+                            SIZE_SUB(part->pos, SZ(4)));
+
+                        part->state = JSON_STR_STATE(plain_char);
+                    }
+                    else {
+                        part->state = JSON_STR_STATE(error);
+                        goto loop_out;
+                    }
+                }
+            }
             break;
 
         default:
@@ -1258,19 +1370,34 @@ parse_out:
         JSON_PARSE_LITERAL_PART(l);                   \
     })
 
-#define JSON_PARSE_LITERAL_BUF_PART(n)                                    \
-    ({                                                                    \
-        obj->current_tok.type = json_ ## n ## _token;                     \
-        obj->current_tok.val.n.buf = &obj->current_tok_buf;               \
-        obj->current_tok.pos = obj->current_pos;                          \
-        obj->current_tok.part.type = json_token_ ## n ## _part;           \
-        obj->current_tok.part.n.state = json_token_ ## n ## _start_state; \
-        json_mem_buf_reset(&obj->current_tok_buf);                        \
-        json_parse_ ## n ## _token(obj, buf);                             \
+#define JSON_PARSE_LITERAL_BUF_PART_INIT_NUMBER()
+
+#define JSON_PARSE_LITERAL_BUF_PART_INIT_STRING()         \
+    ({                                                    \
+        obj->current_tok.part.string.surrogates.high = 0; \
+        obj->current_tok.part.string.surrogates.low = 0;  \
+        obj->current_tok.part.string.pos = 0;             \
     })
 
-#define JSON_PARSE_NUMBER() JSON_PARSE_LITERAL_BUF_PART(number)
-#define JSON_PARSE_STRING() JSON_PARSE_LITERAL_BUF_PART(string)
+#define JSON_PARSE_LITERAL_BUF_PART_INIT(m, n)                            \
+    ({                                                                    \
+        obj->current_tok.part.type = json_token_ ## n ## _part;           \
+        obj->current_tok.part.n.state = json_token_ ## n ## _start_state; \
+        JSON_PARSE_LITERAL_BUF_PART_INIT_ ## m();                         \
+    })
+
+#define JSON_PARSE_LITERAL_BUF_PART(m, n)                   \
+    ({                                                      \
+        obj->current_tok.type = json_ ## n ## _token;       \
+        obj->current_tok.val.n.buf = &obj->current_tok_buf; \
+        obj->current_tok.pos = obj->current_pos;            \
+        json_mem_buf_reset(&obj->current_tok_buf);          \
+        JSON_PARSE_LITERAL_BUF_PART_INIT(m, n);             \
+        json_parse_ ## n ## _token(obj, buf);               \
+    })
+
+#define JSON_PARSE_NUMBER() JSON_PARSE_LITERAL_BUF_PART(NUMBER, number)
+#define JSON_PARSE_STRING() JSON_PARSE_LITERAL_BUF_PART(STRING, string)
 
 static bool json_parse_begin_token(
     struct json_obj_t* obj, struct json_parse_buf_t* buf)
@@ -1791,6 +1918,7 @@ bool json_config_get_param(
     switch (param) {
     case json_allow_literal_value_config:
     case json_allow_multi_objects_config:
+    case json_allow_surrogate_pairs_config:
     case json_disallow_unicode_esc_config:
     case json_disallow_non_ascii_config:
     case json_validate_utf8_config:
@@ -1807,6 +1935,7 @@ void json_config_set_param(
     switch (param) {
     case json_allow_literal_value_config:
     case json_allow_multi_objects_config:
+    case json_allow_surrogate_pairs_config:
     case json_disallow_unicode_esc_config:
     case json_disallow_non_ascii_config:
     case json_validate_utf8_config:
