@@ -47,11 +47,10 @@ const char help[] =
 "  -d|--depth=NUM       depth of generated JSON objects\n"
 "                         (default value: 2)\n"
 "  -e|--expr=EXPR       when action is `-T|--gen-type' and\n"
-"                         the type generated are \"dict\"s,\n"
+"                         the types generated are \"dict\"s,\n"
 "                         generate \"expr\" expressions; EXPR\n"
 "                         is a comma-separated list of strings\n"
-"                       when action is `-V|--gen-value' and\n"
-"                         the type generated are \"dict\"s,\n"
+"                       when action is `-V|--gen-value' do\n"
 "                         generate only the specified keys;\n"
 "                         EXPR is a semicolon-separated list\n"
 "                         of comma-separated list of key names\n"
@@ -69,7 +68,7 @@ const char help[] =
 "                         randomize the order of arguments\n"
 "                         of the generated object\n"
 "  -t|--json-type=TYPE  the kind of json types generated:\n"
-"     --object            either 'o[bject[' or 'd[ict]';\n"
+"     --object            either 'o[bject]' or 'd[ict]';\n"
 "     --dict              the default is 'object'\n"
 "     --dump-options    print options and exit\n"
 "     --version         print version numbers and exit\n"
