@@ -139,8 +139,6 @@ struct VSTACK_TYPE
 
 #define VSTACK_ASSERT_NODE_INVARIANTS(q)  \
     do {                                  \
-        STATIC(TYPEOF_IS_VSTACK(stack));  \
-        STATIC(TYPEOF_IS_VSTACK_POOL(p)); \
         STATIC(TYPEOF_IS_VSTACK_NODE(q)); \
         ASSERT((q)->size > 0);            \
     } while (0)
@@ -323,10 +321,10 @@ static struct VSTACK_NODE_TYPE* VSTACK_PUSH(
     // => p->size - p->used >= s >= t
     ASSERT(p->size - p->used >= t);
 
-    // stack->used + t < stack->max_size:
-    //   t <= s < s + l <= u =>
-    //   t <  u == max_size - used <=>
-    //   used + t < max_size
+    // stack->used + t <= stack->max_size:
+    //   t <= s <= u =>
+    //   t <= u == max_size - used <=>
+    //   used + t <= max_size
     stack->used += t;
 
     r = p->used + a;
