@@ -43,7 +43,7 @@
 
 static const char program[] = STRINGIFY(PROGRAM);
 static const char library[] = STRINGIFY(LIBRARY);
-static const char verdate[] = "0.8.5 -- 2017-05-02 17:59"; // $ date +'%F %R'
+static const char verdate[] = "0.8.6 -- 2017-10-14 10:43"; // $ date +'%F %R'
 
 static const char license[] =
 "Copyright (C) 2016, 2017  Stefan Vargyas.\n"
@@ -464,7 +464,9 @@ static void usage(void)
         "                                 the default is validate\n"
         "  -S|--escape-json             run 'json-escape': produce the JSON-escaped text\n"
         "                                 of the input UTF8 text given\n"
-        "  -J|--json2                   run 'json2' (default)\n"
+        "  -J|--json2                   run 'json2': flatten the hierarchical structure\n"
+        "                                 of the JSON input to a series of text lines of\n"
+        "                                 form /PATH/NAME=VALUE (default)\n"
 //!!!2JSON		"  -2|--2json                 run '2json'\n"
         "the options are:\n"
         "  -d|--type-def TYPE           use the specified type definition or library for\n"
@@ -483,8 +485,15 @@ static void usage(void)
 #include "opt-sizes.def"
         "  -l|--[no-]liter[al]-value    allow literal values on input or otherwise\n"
         "                                 do not (default not)\n"
-        "  -u|--[no-]valid[ate]-utf8    validate non-ascii chars in strings for UTF8\n"
-        "                                 encoding or otherwise not (default not)\n"
+        "  -s|--[no-]surrogate-pairs    allow surrogate pairs in JSON strings (default do not)\n"
+        "                                 when action is anything but `-S|--escape-json';\n"
+        "                                 when action is `-S|--escape-json', produce\n"
+        "                                 surrogate pairs on output (default do not)\n"
+        "                                 when given `-u|--validate-utf8' too\n"
+        "  -u|--[no-]valid[ate]-utf8    validate for UTF8 encoding the non-ascii chars\n"
+        "                                 in JSON strings (when the action given is any but\n"
+        "                                 `-S|--escape-json') or in whole text (when given\n"
+        "                                 `-S|--escape-json'); default: do not validate\n"
         "  -a|--[no-]ascii-only         enforce JSON strings to contain ASCII chars\n"
         "                                 only or otherwise do not (default not)\n"
         "  -m|--[no-]multi-object       allow multiple JSON objects on input or\n"
@@ -781,6 +790,7 @@ static void dump_options(const struct options_t* opts)
         "type-name:                     %s\n"
         "quote-cntrl:                   %s\n"
         "liter-value:                   %s\n"
+        "surrogate-pairs:               %s\n"
         "valid-utf8:                    %s\n"
         "ascii-only:                    %s\n"
         "multi-obj:                     %s\n"
@@ -824,6 +834,7 @@ static void dump_options(const struct options_t* opts)
         NNUL(type_name),
         NOYES(quote_cntrl),
         NOYES(liter_value),
+        NOYES(surrogate_pairs),
         NOYES(valid_utf8),
         NOYES(ascii_only),
         NOYES(multi_obj),
@@ -1187,6 +1198,7 @@ const struct options_t* options(int argc, char* argv[])
         .type_name        = NULL,
         .quote_cntrl      = false,
         .liter_value      = false,
+        .surrogate_pairs  = false,
         .valid_utf8       = false,
         .ascii_only       = false,
         .multi_obj        = false,
@@ -1235,6 +1247,7 @@ const struct options_t* options(int argc, char* argv[])
         input_file_opt       = 'i',
         quote_cntrl_opt      = 'q',
         liter_value_opt      = 'l',
+        surrogate_pairs_opt  = 's',
         valid_utf8_opt       = 'u',
         ascii_only_opt       = 'a',
         multi_obj_opt        = 'm',
@@ -1262,6 +1275,7 @@ const struct options_t* options(int argc, char* argv[])
         no_type_check_opt,
         no_quote_cntrl_opt,
         no_liter_value_opt,
+        no_surrogate_pairs_opt,
         no_valid_utf8_opt,
         no_ascii_only_opt,
         no_multi_obj_opt,
@@ -1312,6 +1326,8 @@ const struct options_t* options(int argc, char* argv[])
         { "literal-value",          0,       0, liter_value_opt },
         { "no-literal-value",       0,       0, no_liter_value_opt },
         { "valid-utf8",             0,       0, valid_utf8_opt },
+        { "surrogate-pairs",        0,       0, surrogate_pairs_opt },
+        { "no-surrogate-pairs",     0,       0, no_surrogate_pairs_opt },
         { "no-valid-utf8",          0,       0, no_valid_utf8_opt },
         { "validate-utf8",          0,       0, valid_utf8_opt },
         { "no-validate-utf8",       0,       0, no_valid_utf8_opt },
@@ -1362,7 +1378,7 @@ const struct options_t* options(int argc, char* argv[])
 #define CASE(type, id, name, pad, letter, desc, help, min, max, def) \
         ARG_LETTER(letter)
 #include "opt-sizes.def"
-        "ad:i:lmN::orqQt:uvV";
+        "ad:i:lmN::orqQst:uvV";
 
     struct bits_opts_t {
         bits_t dump: 1;
@@ -1436,6 +1452,12 @@ const struct options_t* options(int argc, char* argv[])
         case no_liter_value_opt:
             opts.liter_value = false;
             break;
+        case surrogate_pairs_opt:
+            opts.surrogate_pairs = true;
+            break;
+        case no_surrogate_pairs_opt:
+            opts.surrogate_pairs = false;
+            break;
         case valid_utf8_opt:
             opts.valid_utf8 = true;
             break;
