@@ -120,6 +120,7 @@ static void obj_json_base_init_json(
     };
     static const size_t types[] = {
         CONF_DEF(allow_literal_value, obj),
+        CONF_DEF(allow_surrogate_pairs, obj|esc),
         CONF_DEF(disallow_non_ascii, obj),
         CONF_DEF(allow_multi_objects, obj),
         CONF_DEF(validate_utf8, obj|esc),
@@ -128,6 +129,7 @@ static void obj_json_base_init_json(
     };
 
     CONF_SET(allow_literal_value, liter_value);
+    CONF_SET(allow_surrogate_pairs, surrogate_pairs);
     CONF_SET(disallow_non_ascii, ascii_only);
     CONF_SET(allow_multi_objects, multi_obj);
     CONF_SET(validate_utf8, valid_utf8);
@@ -275,7 +277,7 @@ void obj_json_base_done(struct obj_json_base_t* this)
 
 static void pos_error_verbose(
     struct obj_json_base_t* this, const char* file,
-    size_t line, size_t col, size_t offset,
+    const struct json_error_pos_t* pos, size_t offset,
     const uchar_t* buf, size_t len,
     bool verbose, bool at_eof)
 {
@@ -285,7 +287,7 @@ static void pos_error_verbose(
     if (this->no_error)
         return;
 
-    pos_error_header(file, line, col);
+    pos_error_header(file, pos->line, pos->col);
     json_intf_print_error_desc(this->json, stderr);
     fputc('\n', stderr);
 
@@ -293,7 +295,7 @@ static void pos_error_verbose(
     PRINT_DEBUG_BEGIN(
         "err_ctxt=%zu line=%zu col=%zu offset=%zu "
         "verbose=%d at_eof=%d len=%zu buf=[%p]",
-        this->err_ctxt, line, col, offset,
+        this->err_ctxt, pos->line, pos->col, offset,
         verbose, at_eof, len, buf);
     pretty_print_string(
         stderr, buf, len,
@@ -301,7 +303,7 @@ static void pos_error_verbose(
     PRINT_DEBUG_END();
 #endif
 
-    if (!verbose || line == 0 || col == 0)
+    if (!verbose || pos->line == 0 || pos->col == 0)
         return;
 
     ASSERT(this->err_ctxt > 0);
@@ -337,7 +339,7 @@ static void pos_error_verbose(
         n ++;
     }
 
-    pos_error_header(file, line, col);
+    pos_error_header(file, pos->line, pos->col);
     for (p = b; p < e; p ++) {
         const char* f =
             pretty_print_fmt(*p, 0);
@@ -353,7 +355,7 @@ static void pos_error_verbose(
     }
     fputc('\n', stderr);
 
-    pos_error_header(file, line, col);
+    pos_error_header(file, pos->line, pos->col);
     fprintf(stderr, "%*c\n", SIZE_AS_INT(n), '^');
 }
 
@@ -587,7 +589,7 @@ int obj_json_base_run(struct obj_json_base_t* this)
         pos_error_verbose(
             this,
             f ? f->name : this->input,
-            w.line, w.col, k,
+            &w, k,
             f ? f->buf : NULL,
             f ? f->size : 0,
             f && f->buf &&
@@ -882,7 +884,7 @@ int obj_json_base_run(struct obj_json_base_t* this)
         pos_error_verbose(
             this,
             this->input,
-            w.line, w.col, a.error,
+            &w, a.error,
             this->buf->ptr,
             this->buf->len,
             this->verbose,
@@ -891,8 +893,7 @@ int obj_json_base_run(struct obj_json_base_t* this)
         pos_error_verbose(
             this,
             this->input,
-            w.line, w.col,
-            0, NULL, 0,
+            &w, 0, NULL, 0,
             false, false);
 
     if (c)
@@ -934,7 +935,7 @@ bool obj_json_base_check_error(struct obj_json_base_t* this)
         pos_error_verbose(
             this,
             this->input,
-            w.line, w.col, k,
+            &w, k,
             this->buf->ptr,
             this->buf->len,
             this->verbose,
@@ -944,8 +945,7 @@ bool obj_json_base_check_error(struct obj_json_base_t* this)
         pos_error_verbose(
             this,
             this->input,
-            w.line, w.col,
-            0, NULL, 0,
+            &w, 0, NULL, 0,
             false, false);
 
     return true;
