@@ -27,11 +27,9 @@
 #include "json-esc.h"
 
 static void obj_json_escape_init(
-    struct obj_json_escape_t* this, const struct obj_type_t* type,
-    const struct options_t* opts)
+    struct obj_json_escape_t* this, const struct options_t* opts)
 {
-    obj_init(OBJ_SUPER(this), type);
-    OBJ_INIT(this, type);
+    obj_init(OBJ_SUPER(this));
 
     obj_json_base_init_esc(JSON_BASE(this), opts);
 }
