@@ -28,7 +28,7 @@ int main(int argc, char* argv[])
 
     OBJ_DECL(obj, type);
 
-    type->init(obj, type, opts);
+    type->init(obj, opts);
     r = type->run(obj);
     type->done(obj);
 
