@@ -234,11 +234,9 @@ static struct json_handler_t handler = {
 };
 
 static void obj_json2_init(
-    struct obj_json2_t* this, const struct obj_type_t* type,
-    const struct options_t* opts)
+    struct obj_json2_t* this, const struct options_t* opts)
 {
-    obj_init(OBJ_SUPER(this), type);
-    OBJ_INIT(this, type);
+    obj_init(OBJ_SUPER(this));
 
     ASSERT(handler.array_start_func == NULL);
     ASSERT(handler.array_end_func == NULL);
