@@ -33,7 +33,7 @@ struct options_t;
 typedef size_t obj_size_t;
 
 typedef void (*obj_init_func_t)(
-    void*, const struct obj_type_t*, const struct options_t*);
+    void*, const struct options_t*);
 typedef void (*obj_done_func_t)(void*);
 typedef int (*obj_run_func_t)(void*);
 
@@ -57,23 +57,13 @@ enum obj_name_t  {
 
 const struct obj_type_t* get_obj_type(enum obj_name_t);
 
-#define OBJ_TYPE() \
-const struct obj_type_t* type;
-
-#define OBJ_ROOT() \
-OBJ_TYPE();
-
 #define OBJ_BASE(NAME) \
-struct NAME base; \
-OBJ_TYPE();
+struct NAME base
 
 #define OBJ_DECL(OBJ, TYPE)                    \
 char OBJ ## __[TYPE->size + TYPE->align];      \
 char* OBJ = PTR_ALIGN(OBJ ## __, TYPE->align);
 
-#define OBJ_INIT(THIS, TYPE) \
-THIS->type = TYPE;
-
 #define OBJ_SUPER(obj) \
 (&((obj)->base))
 
@@ -88,12 +78,10 @@ const struct obj_type_t NAME = {               \
 
 struct obj_t
 {
-    OBJ_ROOT();
-
     FILE* output;
 };
 
-void obj_init(struct obj_t*, const struct obj_type_t*);
+void obj_init(struct obj_t*);
 void obj_done(struct obj_t*);
 
 #endif/*__OBJ_H */
